@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Code for Life
 #
-# Copyright (C) 2018, Ocado Innovation Limited
+# Copyright (C) 2021, Ocado Innovation Limited
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -34,22 +34,24 @@
 # copyright notice and these terms. You must not misrepresent the origins of this
 # program; modified versions of the program must be marked as such and not
 # identified as the original program.
-from datetime import timedelta
+import datetime
+import json
 from uuid import uuid4
 
-from django.core.mail import EmailMultiAlternatives
-from django.template import loader
-from django.utils import timezone
-from requests import post
-from requests.exceptions import RequestException
-
+from common import app_settings
 from common.email_messages import (
     emailChangeNotificationEmail,
     emailChangeVerificationEmail,
+    emailChangeDuplicateNotificationEmail,
     emailVerificationNeededEmail,
 )
-from common.models import EmailVerification
-from common import app_settings
+from common.models import EmailVerification, Teacher, Student
+from django.core.mail import EmailMultiAlternatives
+from django.http import HttpResponse
+from django.template import loader
+from django.utils import timezone
+from requests import post, get, put
+from requests.exceptions import RequestException
 
 NOTIFICATION_EMAIL = "Code For Life Notification <" + app_settings.EMAIL_ADDRESS + ">"
 VERIFICATION_EMAIL = "Code For Life Verification <" + app_settings.EMAIL_ADDRESS + ">"
@@ -94,7 +96,7 @@ def generate_token(user, email="", preverified=False):
         user=user,
         email=email,
         token=uuid4().hex[:30],
-        expiry=timezone.now() + timedelta(hours=1),
+        expiry=timezone.now() + datetime.timedelta(hours=1),
         verified=preverified,
     )
 
@@ -134,13 +136,47 @@ def is_verified(user):
 
 def add_to_dotmailer(first_name: str, last_name: str, email: str):
     try:
+        create_contact(first_name, last_name, email)
         add_contact_to_address_book(first_name, last_name, email)
     except RequestException:
-        return
+        return HttpResponse(status=404)
+
+
+def create_contact(first_name, last_name, email):
+    url = app_settings.DOTMAILER_CREATE_CONTACT_URL
+    body = {
+        "contact": {
+            "email": email,
+            "optInType": "VerifiedDouble",
+            "emailType": "Html",
+            "dataFields": [
+                {"key": "FIRSTNAME", "value": first_name},
+                {"key": "LASTNAME", "value": last_name},
+                {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
+            ],
+        },
+        "consentFields": [
+            {
+                "fields": [
+                    {
+                        "key": "DATETIMECONSENTED",
+                        "value": datetime.datetime.now().__str__(),
+                    },
+                ]
+            }
+        ],
+        "preferences": app_settings.DOTMAILER_DEFAULT_PREFERENCES,
+    }
+
+    post(
+        url,
+        json=body,
+        auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD),
+    )
 
 
 def add_contact_to_address_book(first_name, last_name, email):
-    url = app_settings.DOTMAILER_URL
+    url = app_settings.DOTMAILER_ADDRESS_BOOK_URL
     body = {
         "email": email,
         "optInType": "VerifiedDouble",
@@ -150,8 +186,8 @@ def add_contact_to_address_book(first_name, last_name, email):
             {"key": "LASTNAME", "value": last_name},
             {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
         ],
-        "preferences": app_settings.DOTMAILER_DEFAULT_PREFERENCES,
     }
+
     post(
         url,
         json=body,
@@ -159,11 +195,85 @@ def add_contact_to_address_book(first_name, last_name, email):
     )
 
 
-def update_email(user, request, data):
+def get_dotmailer_user_by_email(email):
+    url = app_settings.DOTMAILER_GET_USER_BY_EMAIL_URL.replace("EMAIL", email)
+
+    response = get(
+        url,
+        auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD),
+    )
+
+    return json.loads(response.content)
+
+
+def add_consent_record_to_dotmailer_user(user):
+    consent_date_time = datetime.datetime.now().__str__()
+
+    url = app_settings.DOTMAILER_PUT_CONSENT_DATA_URL.replace(
+        "USER_ID", str(user["id"])
+    )
+    body = {
+        "contact": {
+            "email": user["email"],
+            "optInType": user["optInType"],
+            "emailType": user["emailType"],
+            "dataFields": user["dataFields"],
+        },
+        "consentFields": [
+            {
+                "fields": [
+                    {"key": "DATETIMECONSENTED", "value": consent_date_time},
+                ]
+            }
+        ],
+    }
+
+    put(
+        url,
+        json=body,
+        auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD),
+    )
+
+
+def send_dotmailer_consent_confirmation_email_to_user(user):
+    url = app_settings.DOTMAILER_SEND_CAMPAIGN_URL
+    campaign_id = app_settings.DOTMAILER_THANKS_FOR_STAYING_CAMPAIGN_ID
+    body = {
+        "campaignID": campaign_id,
+        "contactIds": [str(user["id"])],
+    }
+
+    post(
+        url,
+        json=body,
+        auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD),
+    )
+
+
+def update_email(user: Teacher or Student, request, data):
     changing_email = False
     new_email = data["email"]
-    if new_email != "" and new_email != user.email:
-        # new email to set and verify
+
+    if new_email != "" and new_email != user.new_user.email:
         changing_email = True
-        send_verification_email(request, user, new_email)
+        if _is_email_already_taken(new_email, user):
+            email_message = emailChangeDuplicateNotificationEmail(request, new_email)
+            send_email(
+                NOTIFICATION_EMAIL,
+                [user.new_user.email],
+                email_message["subject"],
+                email_message["message"],
+            )
+        else:
+            # new email to set and verify
+            send_verification_email(request, user.new_user, new_email)
     return changing_email, new_email
+
+
+def _is_email_already_taken(new_email, user):
+    teachers_with_email = Teacher.objects.filter(new_user__email=new_email)
+    students_with_email = Student.objects.filter(new_user__email=new_email)
+
+    return (teachers_with_email.exists() and teachers_with_email[0] != user) or (
+        students_with_email.exists() and students_with_email[0] != user
+    )
